# -*- coding: UTF-8 -*-
# pep8: disable-msg=E501
# pylint: disable=C0301
import os
import logging
import getpass
import tempfile

__version__ = '{{ packagesample.version }}'
__author__ = 'Your Name'
__author_username__ = 'your_username'
__author_email__ = 'yourname@gmail.com'
__description__ = 'Generated from a template'


log_filename = os.path.join(tempfile.gettempdir(),
                            'packagesample-' + getpass.getuser() + '.log')
log_format = '%(asctime)s %(levelname)-5s '
log_format += '%(filename)+12s:%(lineno)03d %(message)s (%(name)s)'

logging.basicConfig(level=logging.DEBUG,
                    format=log_format,
                    filename=log_filename,
                    filemode='a')
# define a Handler which writes INFO messages or higher to the sys.stderr
console = logging.StreamHandler()
console.setLevel(logging.INFO)
# set a format which is simpler for console use
formatter = logging.Formatter(log_format)
# tell the handler to use this format
console.setFormatter(formatter)
# add the handler to the root logger
logging.getLogger('').addHandler(console)

log = logging.getLogger('packagesample')


def __path(filename):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), filename)

# Jenkins
if os.getenv("BUILD_NUMBER"):
    file_ = open(__path('build.info'), 'w')
    file_.write(os.getenv("BUILD_NUMBER"))
    file_.close()

# Travis
if os.getenv("TRAVIS_BUILD_NUMBER"):
    file_ = open(__path('build.info'), 'w')
    file_.write(os.getenv("TRAVIS_BUILD_NUMBER"))
    file_.close()

__build__ = '0'
if os.path.exists(__path('build.info')):
    __build__ = open(__path('build.info')).read().strip()

__version__ = __version__ + '.' + __build__



# -*- coding: utf-8 -*-
"""
    Setup package
    ~~~~~~~~~~~~~~~~~~~~

    Setuptools/distutils commands to package installation.

    :license: Other/Proprietary, see LICENSE for details.
"""
# pylint: HOOK-IGNORED

import os
from setuptools import setup
from setuptools import find_packages
import subprocess

project_name = 'packagesample'
__version__ = __import__(project_name).__version__
__author__ = __import__(project_name).__author__
__author_email__ = __import__(project_name).__author_email__
__author_username__ = __import__(project_name).__author_username__
__description__ = __import__(project_name).__description__


def cmd_output(args, **kwds):
    kwds.setdefault("stdout", subprocess.PIPE)
    kwds.setdefault("stderr", subprocess.STDOUT)
    p = subprocess.Popen(args, **kwds)
    return p.communicate()[0]


def version_tuple(version):
    return tuple([int(num) for num in version.split('.')])


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except:
        pass
    return ''


CLASSIFIERS = []

setup(
    author=__author__,
    author_email='maglovato@gmail.com',
    classifiers=CLASSIFIERS,
    description=__description__,
    download_url='https://github.com/{}/{}/tarball/{}'.format(
        __author_username__, project_name, __version__),
    entry_points={
        'console_scripts': [
            project_name + '_app = ' + project_name + '.start:main'
        ]
    },
    include_package_data=True,
    install_requires=read('requirements.txt'),
    license=read('LICENSE'),
    long_description=read('README.md'),
    name=project_name,
    packages=find_packages(),
    platforms=['any'],
    scripts=[],
    test_suite='nose.collector',
    tests_require=read('requirements-dev.txt'),
    url='https://github.com/' + __author_username__ + '/' + project_name,
    version=__version__,
    cmdclass={},
    zip_safe=True
)



# -*- coding: utf-8 -*-
# pep8: disable-msg=E501
# pylint: disable=C0301

from machete import __version__
from machete import log
from machete.submodule import module
import argparse
import os


def main():
    log.info("machete v" + __version__)
    parser = argparse.ArgumentParser(description='machete v' + __version__)

    path = os.path.abspath(os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "templates"))
    tpls = [fn for fn in os.listdir(path) if any([not fn.startswith('_')])]

    templates_str = ', '.join(tpls)
    k = templates_str.rfind(",")
    templates_str = templates_str[:k] + " or" + templates_str[k + 1:]

    parser.add_argument(
        '-t', '--template',
        help='Select one of the available templates. \
             Allowed values are: ' + templates_str + '.',
        metavar='<template_name>',
        choices=tpls,
        required=True)
    parser.add_argument(
        "--chicken", help="Chicken mode (optional). Does NOT CHANGE anything.",
        action="store_true")
    args = parser.parse_args()

    try:
        module.main(args.template, args.chicken)
        exit(0)
    except Exception:
        import traceback
        log.error(traceback.format_exc())
        exit(1)



# -*- coding: UTF-8 -*-
# from machete import __version__
from machete import log
# from machete import log_filename
import shutil
# import errno
import os
from os.path import expanduser
import subprocess
# import stat
import tempfile
IGNORE_PATTERNS = ('*.pyc')

is_chicken = False

windows = os.name == 'nt'


def os_call(path):
    log.debug(path)
    proc = subprocess.Popen(path.split(' '), stdout=subprocess.PIPE)
    (out, err) = proc.communicate()
    return out


def replace_infile(lookfor, replace, text_file):
    try:
        infile = open(text_file)
        outfile = open(text_file + '.tmp', 'w')

        # replacements = {'zero':'0', 'temp':'bob', 'garbage':'nothing'}
        replacements = {lookfor: replace}

        for line in infile:
            for src, target in replacements.iteritems():
                line = line.replace(src, target)
            outfile.write(line)
        infile.close()
        outfile.close()
        shutil.move(text_file + '.tmp', text_file)
        return True
    except Exception, e:
        print str(e)
        return False


def copytree(src, dst, ignore=shutil.ignore_patterns(IGNORE_PATTERNS)):
    # print('trying to copy from '+src+' to '+dst)
    if not os.path.exists(dst):
        os.makedirs(dst)
        shutil.copystat(src, dst)
    lst = os.listdir(src)
    if ignore:
        excl = ignore(src, lst)
        lst = [x for x in lst if x not in excl]
    for item in lst:
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            copytree(s, d, ignore)
        else:
            shutil.copy2(s, d)


def copy_files(template):
    print("Copying files from template...")
    try:
        if not is_chicken:
            run_file = os.path.realpath(__file__)
            run_folder = os.path.dirname(run_file)
            path = os.path.join(os.path.dirname(run_folder), "templates")

            base_path = os.path.join(path, "_base")
            template_path = os.path.join(path, template)

            copytree(base_path, '.')
            copytree(template_path, '.')
        return True
    except Exception, e:
        print str(e)
        return False


def git_init():
    print("Initializing GIT...")
    if not is_chicken:
        try:
            os_call('git init')
        except:
            pass  # thats ok, you can sort of live without git


def git_add_commit():
    if not is_chicken:
        try:
            os_call('git add .')
            os_call('git commit . -m "Initial commit"')
        except:
            pass  # thats ok, you can sort of live without git


def rename_files(project):
    print("Modifying copied files names...")
    try:
        if not is_chicken:
            shutil.move('packagesample', project)
        return True
    except Exception, e:
        print str(e)
        return False


def perform_replaces(project):
    print("Modifying copied files content...")
    try:
        if not is_chicken:
            cfiles = []
            for root, dirs, files in os.walk('.'):
                for file in files:
                    if file.endswith('.py'):
                        cfiles.append(os.path.join(root, file))
                    if file.endswith('.rst'):
                        cfiles.append(os.path.join(root, file))
                    if file.endswith('.cfg'):
                        cfiles.append(os.path.join(root, file))
                    if file.endswith('.in'):
                        cfiles.append(os.path.join(root, file))
                    if file.endswith('.venv'):
                        cfiles.append(os.path.join(root, file))
                    if file.endswith('.coveragerc'):
                        cfiles.append(os.path.join(root, file))
            for each in cfiles:
                suffix = '{{ packagesample'
                replace_infile(suffix + ' }}', project, each)
                replace_infile(suffix + '.version }}', '0.0.1', each)
                replace_infile(suffix + '.release_date }}', '2014', each)
                replace_infile(suffix + '.repo_name }}', project, each)
                replace_infile(suffix + '.project_name', 'Tha Project', each)
                replace_infile(suffix + '.full_name', 'Your Name', each)
                replace_infile(suffix + '.year', '2014', each)
                replace_infile('packagesample', project, each)
        return True
    except Exception, e:
        print str(e)
        return False


# def has_virtualenv():
#     try:
#         return 'Usage' in os_call('virtualenv')
#     except:
#         return False


# def has_virtualenvwrapper():
#     if has_virtualenv():
#         try:
#             if not windows:
#                 return '/usr' in os_call('which virtualenvwrapper.sh')
#             else:
#                 return False
#         except:
#             return False
#     else:
#         return False


def create_venv(project):
    print("Creating virtualenv...")
    try:
        if not is_chicken:
            log.debug('Creating virtualenv')

            home_dir = expanduser('~').replace('\\', '/')

            venv_path = os.path.join(home_dir, '.virtualenvs')
            try:
                os.makedirs(venv_path)
            except:
                pass
            full_venv_path = os.path.join(venv_path, project)
            os_call('virtualenv ' + full_venv_path)

            print("Virtualenv created!!!\n")

            print("Installing dependencies...")
            os_call('vex ' + project + ' pip install -r requirements.txt')
            print("Now development depencencies... please wait a bit more.")
            os_call('vex ' + project + ' pip install -r requirements-dev.txt')
            print("Installed!")

        return True
    except Exception, e:
        print str(e)
        return False


def main(template, chicken):
    global is_chicken
    is_chicken = chicken
    if is_chicken:
        log.debug('RUNNING ON CHICKEN MODE')
        print('*** RUNNING ON CHICKEN MODE ***')

    if os.listdir('.'):
        message = 'Current directory is not empty, so machete cannot run :-('
        log.warn(message)
        print(message)
        exit(1)

    project = os.path.basename(os.getcwd()).replace('-', '_')
    # git_init()
    if copy_files(template):
        if rename_files(project):
            if perform_replaces(project):
                if create_venv(project):
                    print('\nIts done!')
                    print('\nRun with "vex ' + project + ' python run.py"')
                    print('Check for the log file under ' +
                          tempfile.gettempdir())



# -*- coding: utf-8 -*-
# pylint: disable-msg=E0611

""" Example test to check version number """

from nose.tools import assert_equals  # @UnresolvedImport
from packagesample import __version__


def test_version():
    """ Base test to version method """

    assert_equals(type(__version__), str)



# -*- coding: UTF-8 -*-
from packagesample import __version__
# from packagesample import __chicken__
from packagesample import log
import __builtin__


def main():
    log.info("hello world packagesample.modules.module v" + __version__)
    log.debug("chicken: " + str(__builtin__.chicken))
    return "hello world packagesample.modules.module v" + __version__
